@@ -13,7 +13,7 @@
 ############################################################################
 """Component and Component Architecture Interfaces
 
-$Id: interfaces.py 72301 2007-02-01 17:03:58Z aaron $
+$Id: interfaces.py 88794 2008-07-24 16:17:58Z wichert $
 """
 
 import zope.deferredimport
@@ -512,6 +512,7 @@ class IUtilityRegistration(IRegistration):
     """Information about the registration of a utility
     """
 
+    factory = interface.Attribute("The factory used to create the utility. Optional.")
     component = interface.Attribute("The object registered")
     provided = interface.Attribute("The interface provided by the component")
 
@@ -571,11 +572,11 @@ class Registered(RegistrationEvent):
     interface.implements(IRegistered)
 
 class IUnregistered(IRegistrationEvent):
-    """A component or factory was registered
+    """A component or factory was unregistered
     """
 
 class Unregistered(RegistrationEvent):
-    """A component or factory was registered
+    """A component or factory was unregistered
     """
     interface.implements(IUnregistered)
 
@@ -583,9 +584,12 @@ class IComponentRegistry(interface.Interface):
     """Register components
     """
 
-    def registerUtility(component, provided=None, name=u'', info=u''):
+    def registerUtility(component=None, provided=None, name=u'', info=u'', factory=None):
         """Register a utility
 
+        factory
+           Factory for the component to be registerd.
+
         component
            The registered component
 
@@ -602,10 +606,11 @@ class IComponentRegistry(interface.Interface):
            An object that can be converted to a string to provide
            information about the registration.
 
+        Only one of component and factory can be used.
         A Registered event is generated with an IUtilityRegistration.
         """
 
-    def unregisterUtility(component=None, provided=None, name=u''):
+    def unregisterUtility(component=None, provided=None, name=u'', factory=None):
         """Unregister a utility
 
         A boolean is returned indicating whether the registry was
@@ -614,6 +619,9 @@ class IComponentRegistry(interface.Interface):
         None and is not registered, then the function returns
         False, otherwise it returns True.
 
+        factory
+           Factory for the component to be unregisterd.
+
         component
            The registered component The given component can be
            None, in which case any component registered to provide
@@ -629,11 +637,12 @@ class IComponentRegistry(interface.Interface):
         name
            The utility name.
 
+        Only one of component and factory can be used.
         An UnRegistered event is generated with an IUtilityRegistration.
         """
 
     def registeredUtilities():
-        """Return an iterable of IUtilityRegistrations.
+        """Return an iterable of IUtilityRegistration instances.
 
         These registrations describe the current utility registrations
         in the object.
@@ -720,7 +729,7 @@ class IComponentRegistry(interface.Interface):
         """
 
     def registeredAdapters():
-        """Return an iterable of IAdapterRegistrations.
+        """Return an iterable of IAdapterRegistration instances.
 
         These registrations describe the current adapter registrations
         in the object.
@@ -816,7 +825,7 @@ class IComponentRegistry(interface.Interface):
         """
 
     def registeredSubscriptionAdapters():
-        """Return an iterable of ISubscriptionAdapterRegistrations.
+        """Return an iterable of ISubscriptionAdapterRegistration instances.
 
         These registrations describe the current subscription adapter
         registrations in the object.
@@ -900,7 +909,7 @@ class IComponentRegistry(interface.Interface):
         """
 
     def registeredHandlers():
-        """Return an iterable of IHandlerRegistrations.
+        """Return an iterable of IHandlerRegistration instances.
 
         These registrations describe the current handler registrations
         in the object.
