@@ -13,7 +13,7 @@
 ##############################################################################
 """Basic components support
 
-$Id: registry.py 76703 2007-06-14 23:44:49Z rossp $
+$Id: registry.py 88810 2008-07-25 12:28:28Z wichert $
 """
 
 import types
@@ -65,23 +65,28 @@ class Components(object):
         lambda self, bases: self._setBases(bases),
         )
 
-    def registerUtility(self, component, provided=None, name=u'', info=u'',
-                        event=True):
+    def registerUtility(self, component=None, provided=None, name=u'', info=u'',
+                        event=True, factory=None):
+        if factory:
+            if component:
+                raise TypeError("Can't specify factory and component.")
+            component = factory()
+
         if provided is None:
             provided = _getUtilityProvided(component)
 
-        if (self._utility_registrations.get((provided, name))
-            == (component, info)):
+        reg = self._utility_registrations.get((provided, name))
+        if reg is not None and reg[:2] == (component, info):
             # already registered
             return
 
-        subscribed = [
-            1
-            for ((p, _), (c,_)) in self._utility_registrations.iteritems()
-            if p == provided and c == component
-            ]
+        subscribed = False
+        for ((p, _), data) in self._utility_registrations.iteritems():
+            if p == provided and data[0] == component:
+                subscribed = True
+                break
 
-        self._utility_registrations[(provided, name)] = component, info
+        self._utility_registrations[(provided, name)] = component, info, factory
         self.utilities.register((), provided, name, component)
 
         if not subscribed:
@@ -89,13 +94,18 @@ class Components(object):
 
         if event:
             zope.event.notify(interfaces.Registered(
-                UtilityRegistration(self, provided, name, component, info)
+                UtilityRegistration(self, provided, name, component, info, factory)
                 ))
 
-    def unregisterUtility(self, component=None, provided=None, name=u''):
+    def unregisterUtility(self, component=None, provided=None, name=u'', factory=None):
+        if factory:
+            if component:
+                raise TypeError("Can't specify factory and component.")
+            component = factory()
+
         if provided is None:
             if component is None:
-                raise TypeError("Must specify one of component and provided")
+                raise TypeError("Must specify one of component, factory and provided")
             provided = _getUtilityProvided(component)
 
         old = self._utility_registrations.get((provided, name))
@@ -108,25 +118,25 @@ class Components(object):
         del self._utility_registrations[(provided, name)]
         self.utilities.unregister((), provided, name)
 
-        subscribed = [
-            1
-            for ((p, _), (c,_)) in self._utility_registrations.iteritems()
-            if p == provided and c == component
-            ]
+        subscribed = False
+        for ((p, _), data) in self._utility_registrations.iteritems():
+            if p == provided and data[0] == component:
+                subscribed = True
+                break
 
         if not subscribed:
             self.utilities.unsubscribe((), provided, component)
 
         zope.event.notify(interfaces.Unregistered(
-            UtilityRegistration(self, provided, name, component, old[1])
+            UtilityRegistration(self, provided, name, component, *old[1:])
             ))
 
         return True
 
     def registeredUtilities(self):
-        for ((provided, name), (component, info)
+        for ((provided, name), data
              ) in self._utility_registrations.iteritems():
-            yield UtilityRegistration(self, provided, name, component, info)
+            yield UtilityRegistration(self, provided, name, *data)
 
     def queryUtility(self, provided, name=u'', default=None):
         return self.utilities.lookup((), provided, name, default)
@@ -398,18 +408,20 @@ class UtilityRegistration(object):
 
     interface.implements(interfaces.IUtilityRegistration)
 
-    def __init__(self, registry, provided, name, component, doc):
-        (self.registry, self.provided, self.name, self.component, self.info
-         ) = registry, provided, name, component, doc
+    def __init__(self, registry, provided, name, component, doc, factory=None):
+        (self.registry, self.provided, self.name, self.component, self.info,
+         self.factory
+         ) = registry, provided, name, component, doc, factory
 
     def __repr__(self):
-        return '%s(%r, %s, %r, %s, %r)' % (
-            self.__class__.__name__,
-            self.registry,
-            getattr(self.provided, '__name__', None), self.name,
-            getattr(self.component, '__name__', `self.component`), self.info,
-            )
-
+        return '%s(%r, %s, %r, %s, %r, %r)' % (
+                self.__class__.__name__,
+                self.registry,
+                getattr(self.provided, '__name__', None), self.name,
+                getattr(self.component, '__name__', `self.component`),
+                self.factory, self.info,
+                )
+        
     def __cmp__(self, other):
         return cmp(self.__repr__(), other.__repr__())
 
